/* -*- c++ -*- */
/*
 * Copyright 2024 Daniel Estevez <daniel@destevez.net>
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

#ifndef INCLUDED_DIGITAL_ADDITIVE_SCRAMBLER_BB_H
#define INCLUDED_DIGITAL_ADDITIVE_SCRAMBLER_BB_H

// This file exists only to maintain a backwards-compatible API

#include <gnuradio/digital/additive_scrambler.h>

#endif /* INCLUDED_DIGITAL_ADDITIVE_SCRAMBLER_BB_H */
